/* packet-wsp.c
 * Routines for Wireless Session Protocol dissection
 * Copyright 2000, B.P. Fowler <BPFowler@leedsnet.uklinux.net>
 *
 * $Id: README.developer,v 1.7 2000/03/03 06:58:28 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@unicom.net>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif

#ifdef NEED_SNPRINTF_H
# ifdef HAVE_STDARG_H
#  include <stdarg.h>
# else
#  include <varargs.h>
# endif
# include "snprintf.h"
#endif

#include <string.h>
#include <glib.h>
#include "packet.h"
#include "packet-wap.h"

static const true_false_string continue_truth = { 
	"TPI Present" ,
	"No TPI"
};

static const true_false_string RID_truth = { 
	"Re-Transmission",
	"First transmission"
};

static const true_false_string TIDNew_truth = { 
	"TID is new" ,
	"TID is valid"
};

static const true_false_string UP_truth = { 
	"User Acknowledgement required" ,
	"User Acknowledgement optional"
};

static const value_string vals_pdu_type[] = {
	{ 0, "Not Allowed" },
	{ 1, "Invoke" },
	{ 2, "Result" },
	{ 3, "Ack" },
	{ 4, "Abort" },
	{ 5, "SegmentedInvoke" },
	{ 6, "Segmented Result" },
	{ 6, "Negative Ack" },
	{ 0,       NULL },
};

static const value_string vals_transmission_trailer[] = {
	{ 0, "Not last packet" },
	{ 1, "Last packet of message" },
	{ 2, "Last packet of group" },
	{ 3, "Re-assembly not supported" },
	{ 0,       NULL },
};

static const value_string vals_version[] = {
	{ 0, "Current" },
	{ 1, "Undefined" },
	{ 2, "Undefined" },
	{ 3, "Undefined" },
	{ 0,       NULL },
};

/* File scoped variables for the protocol and registered fields */
static int proto_wsp 				= HF_EMPTY;

/* These fields used by fixed part of header */
static int hf_wsp_header_fixed_part 		= HF_EMPTY;
static int hf_wsp_header_flag_continue 		= HF_EMPTY;
static int hf_wsp_header_pdu_type 		= HF_EMPTY;
static int hf_wsp_header_flag_Trailer 		= HF_EMPTY;
static int hf_wsp_header_flag_RID 		= HF_EMPTY;
static int hf_wsp_header_flag_TID 		= HF_EMPTY;

/* These fields used by Invoke packets */
static int hf_wsp_header_Inv_version 		= HF_EMPTY;
static int hf_wsp_header_Inv_flag_TIDNew 	= HF_EMPTY;
static int hf_wsp_header_Inv_flag_UP	 	= HF_EMPTY;
static int hf_wsp_header_Inv_Reserved	 	= HF_EMPTY;
static int hf_wsp_header_Inv_TransactionClass 	= HF_EMPTY;


static int hf_wsp_header_variable_part 		= HF_EMPTY;
static int hf_wsp_data 				= HF_EMPTY;

/* Initialize the subtree pointers */
static gint ett_wsp 			= ETT_EMPTY;
static gint ett_header 			= ETT_EMPTY;

/* Declarations */
static char transaction_class(unsigned char octet);
static char pdu_type(unsigned char octet);
static char retransmission_indicator(unsigned char octet);


/* Code to actually dissect the packets */
void
dissect_wsp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
{
	/* bytes at offset 0 - 3 */
	unsigned char  b0 = pd[offset + 0];

	/* continuation flag */

/* Set up structures we will need to add the protocol subtree and manage it */
	proto_item *ti;
	proto_tree *wsp_tree;
	
/* This field shows up as the "Info" column in the display; you should make
   it, if possible, summarize what's in the packet, so that a user looking
   at the list of packets can tell what type of packet it is.
   "col_add_fstr()" can be used instead of "col_add_str()"; it takes
   "printf()"-like arguments. */

	fprintf( stderr, "dissect_wsp: fd->cinfo is %p\n", fd->cinfo ); 
	if (check_col(fd, COL_INFO)) 
		col_add_str(fd, COL_INFO, "W S P" );

/* In the interest of speed, if "tree" is NULL, don't do any work not
   necessary to generate protocol tree items. */
	if (tree) {
/* NOTE: The offset and length values in the previous call to
   "proto_tree_add_item()" define what data bytes to highlight in the hex
   display window when the line in the protocol tree display
   corresponding to that item is selected.

   END_OF_FRAME is a handy way to highlight all data from the offset to
   the end of the packet. */
		ti = proto_tree_add_item(tree, proto_wsp, NullTVB, offset, END_OF_FRAME, NULL);
	        wsp_tree = proto_item_add_subtree(ti, ett_wsp);

/* Code to process the packet goes here */
	}
}

/* Register the protocol with Ethereal */
void
proto_register_wsp(void)
{                 

/* Setup list of header fields */
	
/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_wsp,
		&ett_header,
	};

/* Register the protocol name and description */
	proto_wsp = proto_register_protocol(
		"Wireless Session Protocol",   	/* protocol name for use by ethereal */ 
		"wsp"                    	/* Abbreviated protocol name, should Match IANA 
						    < URL:http://www.isi.edu/in-notes/iana/assignments/port-numbers/ >
						  */
	);

/* Required function calls to register the header fields and subtrees used 
	proto_register_field_array(proto_wsp, hf, array_length(hf)); */
	proto_register_subtree_array(ett, array_length(ett));
};
